package magma.tools.protocolTester;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JToolBar;

import magma.util.connection.ConnectionException;
import magma.util.connection.impl.ServerConnection;
import magma.util.observer.IObserver;

public class ProtocolTester extends JFrame implements IObserver<byte[]> {
	private static final long serialVersionUID = 1L;

	private static final String SCENE_MSG = "(scene NAO_Type_Zero)";

	private static final String INIT_MSG = "(init (unum 5) (teamname magmaOffenburg))";

	private ServerConnection serverConnection;

	private Thread receiverThread;

	private JMenuBar menuBar;

	private JTextField serverTextField;

	private JCheckBox verbosityCeckBox;

	private JButton runBtn;

	private JButton sendInitBtn;

	private JButton sendBtn;

	private JTextField msgTextField;

	private JTextArea logArea;

	private JToolBar messageBar;

	private JLabel statusLbl;

	private int sendCnt;

	private int recvCnt;

	public ProtocolTester() {
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setTitle("magma Protocol Tester");

		setLayout(new BorderLayout());

		sendCnt = 0;
		recvCnt = 0;

		menuBar = new JMenuBar();

		menuBar.add(new JLabel("Server:"));

		serverTextField = new JTextField("127.0.0.1:3100");
		serverTextField.setMinimumSize(new Dimension(200, 15));
		menuBar.add(serverTextField);

		verbosityCeckBox = new JCheckBox("Verbose");
		// verbosityCeckBox.setSelected(true);
		menuBar.add(verbosityCeckBox);

		runBtn = new JButton("Connect");
		runBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				run();
			}
		});
		menuBar.add(runBtn);

		messageBar = new JToolBar();
		messageBar.setFloatable(false);

		sendInitBtn = new JButton("Send init");
		sendInitBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				send(SCENE_MSG + " " + INIT_MSG + " (beam 1 1 45.0)");
			}
		});
		messageBar.add(sendInitBtn);

		sendBtn = new JButton("Send");
		sendBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				send(msgTextField.getText());
			}
		});
		msgTextField = new JTextField("(syn)");
		msgTextField.setMinimumSize(new Dimension(200, 15));
		messageBar.add(msgTextField);
		messageBar.add(sendBtn);

		JPanel messagePanel = new JPanel();
		messagePanel.setLayout(new BorderLayout());
		messagePanel.add(new JLabel("Send Log:"), BorderLayout.PAGE_START);

		logArea = new JTextArea();
		logArea.setMinimumSize(new Dimension(300, 300));
		logArea.setBorder(BorderFactory.createMatteBorder(1, 0, 1, 0,
				Color.GRAY));
		messagePanel.add(logArea, BorderLayout.CENTER);

		statusLbl = new JLabel();
		updateStatus();

		setJMenuBar(menuBar);

		add(messageBar, BorderLayout.PAGE_START);
		add(messagePanel, BorderLayout.CENTER);
		add(statusLbl, BorderLayout.PAGE_END);

		setSize(500, 350);
		setLocationRelativeTo(null);

		updateControls();
	}

	private void run() {
		if (receiverThread == null) {
			if (serverTextField.getText().contains(":")) {
				String[] addr = serverTextField.getText().split(":");
				if (addr.length == 2) {
					String host = addr[0];
					int port = Integer.parseInt(addr[1]);

					serverConnection = new ServerConnection(host, port);
					serverConnection.attach(this);

					try {
						System.out
								.println("==================== Starting client ====================");
						logArea.setText("");
						sendCnt = 0;
						recvCnt = 0;
						updateStatus();

						System.out.print("Establish connection... ");
						serverConnection.establishConnection();
						System.out.println("done!");
						serverTextField.setEnabled(false);
						runBtn.setText("Disconnect");
						receiverThread = new Thread() {
							@Override
							public void run() {
								try {
									System.out
											.println("Starting receive loop.");
									serverConnection.startReceiveLoop();
								} catch (ConnectionException e) {
									e.printStackTrace();
								}
							};
						};
						receiverThread.start();
						updateControls();
					} catch (ConnectionException e1) {
						e1.printStackTrace();
						serverConnection = null;
					}
				} else {
					System.out.println("Malformated server address: \"" + addr
							+ "\" expecting the format \"ip:port\"");
				}
			}
		} else {
			System.out
					.println("==================== Stopping client ====================");
			serverConnection.stopReceiveLoop();
			serverConnection.closeConnection(); // Hard shut down, since
												// stopReceiveLoop still waits
												// for
												// the next complete message to
												// arrive before shutting down
			System.out.print("Waiting for connection to close down...");
			while (receiverThread.isAlive()) {
				try {
					Thread.sleep(100);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
			System.out.println(" done!");
			System.out.println("Clinet disconnected!");
			serverConnection = null;
			receiverThread = null;

			updateControls();
			serverTextField.setEnabled(true);
			runBtn.setText("Connect");
		}
	}

	private void send(String msg) {
		sendCnt++;
		updateStatus();
		logArea.append(msg + "\n");
		try {
			serverConnection.sendMessage(msg.getBytes());
		} catch (ConnectionException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void update(byte[] content) {
		// Called on receiving a new message content
		recvCnt++;
		updateStatus();
		String msg = new String(content);
		if (verbosityCeckBox.isSelected()) {
			System.out.println(msg);
		}
	}

	private void updateStatus() {
		statusLbl.setText("Send: " + sendCnt + " Recv: " + recvCnt);
	}

	private void updateControls() {
		boolean enabled = receiverThread != null;

		sendInitBtn.setEnabled(enabled);
		sendBtn.setEnabled(enabled);
	}

	public static void main(String[] args) {
		ProtocolTester tester = new ProtocolTester();
		tester.setVisible(true);
	}

}
