/*******************************************************************************
 * Copyright 2008, 2012 Hochschule Offenburg
 * Klaus Dorer, Mathias Ehret, Stefan Glaser, Thomas Huber, Fabian Korak,
 * Simon Raffeiner, Srinivasa Ragavan, Thomas Rinklin,
 * Joachim Schilling, Ingo Schindler, Rajit Shahi, Bjoern Weiler
 *
 * This file is part of magmaOffenburg.
 *
 * magmaOffenburg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * magmaOffenburg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with magmaOffenburg. If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package magma.util.connection;

import magma.util.observer.ISubscribe;

/**
 * Server connection interface
 * 
 * @author Klaus Dorer
 */
public interface IServerConnection extends ISubscribe<byte[]>
{
	/** the default IP of the RoboCup server */
	String SERVER_IP = "127.0.0.1";

	/** Default IP of remote server 141.79.65.126 */
	String REMOTE_SERVER_IP = "141.79.65.126";

	/** the default port of the server for agent connections */
	int SERVER_PORT = 3100;

	/** the default port of the server for monitor and trainer connections */
	int MONITOR_PORT = 3200;

	/**
	 * Creates a connection to the server and initializes the input and output
	 * stream
	 */
	void establishConnection() throws ConnectionException;

	/**
	 * Sends the passed message to the server
	 * @param msg the message to send
	 */
	void sendMessage(byte[] msg) throws ConnectionException;

	/**
	 * Starts the receive loop that only ends once the socket is shutdown
	 * @throws ConnectionException if any connection problem occurred
	 */
	void startReceiveLoop() throws ConnectionException;

	/**
	 * Stops the loop of receiving messages and notifying observers after
	 * receiving the next message
	 */
	void stopReceiveLoop();

	/**
	 * @return true if we are currently connected to the server
	 */
	boolean isConnected();
}